import datetime
import os

import arcpy
import pandas as pd

# Description: Generates the species overlap summarized by HUC2 used for aquatic species

# This script has been approved for release by the U.S. Environmental Protection Agency (USEPA). Although
# the script has been subjected to rigorous review, the USEPA reserves the right to update the script as needed
# pursuant to further analysis and review. No warranty, expressed or implied, is made by the USEPA or the U.S.
# Government as to the functionality of the script and related material nor shall the fact of release constitute
# any such warranty. Furthermore, the script is released on condition that neither the USEPA nor the U.S. Government
# shall be held liable for any damages resulting from its authorized or unauthorized use.


# User input variables
# directory with zone look up tables
look_up_fc_ab = r'path look ups'
# path species grid files used as inputs for overlap
in_directory_grids = r'path\Grids_byProjection'
# out locations
out_path = r'outpath\Range'  # Range or Critical Critical Habitat
# master species list
master_list = r"\MasterListESA_Dec2018_June2020.csv"

# Columns from the master species list that should be included in the output tables
col_include_output = ['EntityID', 'Common Name', 'Scientific Name', 'Status', 'pop_abbrev', 'family', 'Lead Agency',
                      'country', 'Group', 'Des_CH', 'CH_GIS', 'WoE Summary Group', 'Critical_Habitat_YesNo',
                      'Migratory',
                      'Migratory_YesNo', 'CH_Filename', 'Range_Filename', 'L48/NL48']

start_time = datetime.datetime.now()
print "Start Time: " + start_time.ctime()

# Get date
today = datetime.datetime.today()
date = today.strftime('%Y%m%d')

if not os.path.exists(out_path):
    os.mkdir(out_path)
species_df = pd.read_csv(master_list, dtype=object)
[species_df.drop(m, axis=1, inplace=True) for m in species_df.columns.values.tolist() if m.startswith('Unnamed')]
base_sp_df = species_df.loc[:, col_include_output]

list_folder = os.listdir(in_directory_grids)
list_fc_ab = os.listdir(look_up_fc_ab)

for folder in list_folder:
    out_df = pd.DataFrame()
    arcpy.env.workspace = (in_directory_grids + os.sep + folder)
    list_species = arcpy.ListRasters()
    print list_species
    for species in list_species:
        lookup_csv = [t for t in list_fc_ab if t.startswith(species.split("_")[0].upper()
                                                            + "_" + species.split("_")[1].capitalize())]
        lookup_df = pd.read_csv(look_up_fc_ab + os.sep + lookup_csv[0], dtype=object)
        if not arcpy.Raster(in_directory_grids + os.sep + folder + os.sep + species).hasRAT:
            arcpy.BuildRasterAttributeTable_management(in_directory_grids + os.sep + folder + os.sep + species)
        par_zone_array = arcpy.da.TableToNumPyArray(in_directory_grids + os.sep + folder + os.sep + species,
                                                    ['VALUE', 'COUNT'])

        par_zone_df = pd.DataFrame(data=par_zone_array, dtype=object)
        par_zone_df['VALUE'] = par_zone_df['VALUE'].map(lambda n: str(n).split('.')[0]).astype(str)
        lookup_df['HUCID'] = lookup_df['HUCID'].map(lambda n: str(n).split('.')[0]).astype(str)
        merge_huc = pd.merge(par_zone_df, lookup_df, how='outer', left_on='VALUE', right_on='HUCID')
        copy_df = merge_huc[['EntityID', 'HUC2_AB', 'COUNT', 'STUSPS']].copy()
        copy_df = copy_df.loc[copy_df['COUNT'] > 0]

        df_sum = copy_df.groupby(['EntityID', 'HUC2_AB', 'STUSPS'], as_index=False).sum().reset_index()
        out_df = pd.concat([out_df, df_sum])
    out_df['HUC 2 surrogate'] = out_df['HUC2_AB'].map(lambda (n): '20' if str(n).startswith('22') else str(n)).astype(
        str)
    out_df.to_csv(out_path + os.sep + folder + "_HUC2.csv")
    print ("Exported {0}".format(out_path + os.sep + folder + "_HUC2.csv"))

end = datetime.datetime.now()
print "End Time: " + end.ctime()
elapsed = end - start_time
print "Elapsed  Time: " + str(elapsed)
